#include "cAEnManager.hpp"

#include "../../Platform/Hardware_Abstraction/imBasicIO.hpp"
using namespace Platform::Hardware_Abstraction;

namespace AEn
{
	cAEnManager::cAEnManager(const uint32_t ObjectID) :
		mObjectID{ ObjectID },
		mDiagnostics{ mObjectID_mDiagnostics, this },
		mErrorLogbook{ mObjectID_mErrorLogbook,this },
		mN{ mObjectID_mN }
	{
		showValue("\ncAEnManager::cAEnManager - Object ID = ", mObjectID);
	}

	cAEnManager::~cAEnManager()
	{
		showValue("\ncAEnManager::~cAEnManager");
	}

	void cAEnManager::create(void)
	{
		showValue("\ncAEnManager::create");
	}

	void cAEnManager::init(void)
	{
		showValue("\ncAEnManager::init");
		mN.init();
	}

	void cAEnManager::deinit(void)
	{
		showValue("\ncAEnManager::deinit");
		mN.deinit();
	}

	void cAEnManager::buildRelations(void)
	{
		showValue("\ncAEnManager::buildRelations");
		
		mN.registerErrorHandler(this);
	}

	void cAEnManager::resetRelations(void)
	{
		showValue("\ncAEnManager::resetRelations");

		mN.registerErrorHandler(nullptr);
	}

	void cAEnManager::config(uint32_t const Data)
	{
		showValue("\ncAEnManager::config");

		mN.config(Data);
	}

	void cAEnManager::allocateResources(void)
	{
		showValue("\ncAEnManager::allocateResources");
	}

	void cAEnManager::deallocateResources(void)
	{
		showValue("\ncAEnManager::deallocateResources");
	}

	void cAEnManager::start(void)
	{
		showValue("\ncAEnManager::start");
	}

	void cAEnManager::taskInit(void)
	{
		showValue("\nccAEnManager::taskInit");
	}

	void cAEnManager::taskOperational(void)
	{
		showValue("\ncAEnManager::taskOperational");
	}

	void cAEnManager::restart(void)
	{
		showValue("\ncAEnManager::restart");
	}

	void cAEnManager::shutdown(void)
	{
		showValue("\ncAEnManager::shutdown");
	}

	void cAEnManager::registerErrorHandler(icbcErrorHandler* const ptrNextLevelErrorHandler)
	{
		showValue("\ncAEnManager::registerErrorHandler");
		
		mPtrUpperLevelErrorHandler = ptrNextLevelErrorHandler;
	}

	void cAEnManager::handleError(void)
	{
		showValue("\ncAEnManager::handleError");
	}

	void cAEnManager::recoverError(void)
	{
		showValue("\ncAEnManager::recoverError");
	}

	void cAEnManager::enterFailSafe(void)
	{
		showValue("\ncAEnManager::enterFailSafe");
	}

	void cAEnManager::notifyError(const uint32_t ObjectID, const Common::Management::ecError Error)
	{
		showValue("\ncAEnManager::notifyError");
		
		if (Error == Common::Management::ecError::Error_1)
		{
			showValue("\ncAEnManager handles error:");
			Common::Management::cErrorEntry locErrorEntry{ ObjectID, Error };
			mErrorLogbook.addErrorEntry(locErrorEntry);
			mErrorLogbook.show();
		}
		else
		{
			mPtrUpperLevelErrorHandler->notifyError(ObjectID, Error);
		}
	}
}
