#ifndef __cAEnManager_HPP__
#define __cAEnManager_HPP__

#include "../Common/Management/icbcErrorHandler.hpp"
#include "../Common/Management/tcDiagnostics.hpp"
#include "../Common/Management/cErrorLogbook.hpp"
#include "../AEn/cN.hpp"

namespace AEn
{
	class cAEnManager : public Common::Management::icbcErrorHandler
	{
		private:
			static const uint32_t mMaxNumberOfDiagnosticEntries = 4;

		public:
			explicit cAEnManager(const uint32_t ObjectID);
			virtual ~cAEnManager() override;

			void create(void);
			void init(void);
			void deinit(void);
			void buildRelations(void);
			void resetRelations(void);
			void config(uint32_t const Data);
			void allocateResources(void);
			void deallocateResources(void);
			void start(void);
			void taskInit(void);
			void taskOperational(void);

			Common::Management::tcDiagnostics<mMaxNumberOfDiagnosticEntries> executeDiagnostics(void)
			{
				showValue("\ncAEnManager::executeDiagnostics");

				mDiagnostics.write(0, mN.getDiagnostics());

				showValue("\nError logbook AEnManager:");
				mErrorLogbook.show();

				return mDiagnostics;
			}

			constexpr static uint32_t getMaxNumberOfDiagnosticEntries(void)
			{
				return mMaxNumberOfDiagnosticEntries;
			}

			void restart(void);
			void shutdown(void);
			void registerErrorHandler(icbcErrorHandler* const ptrNextLevelErrorHandler);
			void enterFailSafe(void);

		private:
			void notifyError(const uint32_t ObjectID, const Common::Management::ecError Error) override;
			void handleError(void);
			void recoverError(void);

		private:
			static const uint32_t mObjectID_mDiagnostics = 5;
			static const uint32_t mObjectID_mErrorLogbook = 6;
			static const uint32_t mObjectID_mN = 7;

			icbcErrorHandler* mPtrUpperLevelErrorHandler{ nullptr };
			const uint32_t mObjectID;
			Common::Management::tcDiagnostics<mMaxNumberOfDiagnosticEntries> mDiagnostics;
			Common::Management::cErrorLogbook mErrorLogbook;
			AEn::cN mN;
	};
}

#endif  // __cAEnManager_HPP__
