#ifndef __tcDiagnostics_HPP__
#define __tcDiagnostics_HPP__

#include "cDiagnosticsEntry.hpp"

#include "../../Platform/Hardware_Abstraction/imBasicIO.hpp"
using namespace Platform::Hardware_Abstraction;

#include <array>

namespace Common::Management
{
	class icbcErrorHandler;
	
	template<uint32_t MaxNumberOfDiagnosticsEntries>
	class tcDiagnostics
	{
		public:
			explicit tcDiagnostics(const uint32_t ObjectID, Common::Management::icbcErrorHandler* const ptrErrorHandler);
			~tcDiagnostics();

			void write(const uint32_t Index, const cDiagnosticsEntry DiagnosticsEntry);
			cDiagnosticsEntry read(uint32_t const Index) const;

			void show(void) const;

		private:
			const uint32_t mObjectID;
			Common::Management::icbcErrorHandler* mPtrErrorHandler{ nullptr };
			std::array<cDiagnosticsEntry, MaxNumberOfDiagnosticsEntries> mDiagnosticsContainer{ };
	};

	template<uint32_t MaxNumberOfDiagnosticsEntries>
	tcDiagnostics<MaxNumberOfDiagnosticsEntries>::tcDiagnostics(const uint32_t ObjectID, Common::Management::icbcErrorHandler* const ptrErrorHandler) :
		//cCounter{ },
		mObjectID{ ObjectID },
		mPtrErrorHandler{ ptrErrorHandler }
	{
		showValue("\nCommon::Management::cDiagnostics - Object ID = ", mObjectID);
		//++::globObjectID;
	}

	template<uint32_t MaxNumberOfDiagnosticsEntries>
	tcDiagnostics<MaxNumberOfDiagnosticsEntries>::~tcDiagnostics()
	{		
		showValue("\nCommon::Management::~cDiagnostics");
		//--::globObjectID;
	}

	template<uint32_t MaxNumberOfDiagnosticsEntries>
	void tcDiagnostics<MaxNumberOfDiagnosticsEntries>::write(const uint32_t Index, const cDiagnosticsEntry DiagnosticsEntry)
	{
		if (Index < MaxNumberOfDiagnosticsEntries)
		{
			mDiagnosticsContainer[Index] = DiagnosticsEntry;
		}
	}

	template<uint32_t MaxNumberOfDiagnosticsEntries>
	cDiagnosticsEntry tcDiagnostics<MaxNumberOfDiagnosticsEntries>::read(const uint32_t Index) const
	{
		return mDiagnosticsContainer[Index];
	}

	template<uint32_t MaxNumberOfDiagnosticsEntries>
	void tcDiagnostics<MaxNumberOfDiagnosticsEntries>::show(void) const
	{
		for (auto&& refElement : mDiagnosticsContainer)
		{
			showValue("\nObject ID: ", refElement.getDiagnosticsObjectID(),
				      " -> Diagnostics: ", refElement.getDiagnosticsData());
		}
	}
}

#endif  // __tcDiagnostics_HPP__
