#include "cSoftwareManager.hpp"

#include "../../Platform/Hardware_Abstraction/imBasicIO.hpp"
using namespace Platform::Hardware_Abstraction;

namespace Software_Management
{
	cSoftwareManager::cSoftwareManager(void) :
		mObjectID{ mObjectID_SystemManager },
		mDataBase{ mObjectID_mDataBase, this },
		mDiagnostics{ mObjectID_mDiagnostics, this },
		mErrorLogbook{ mObjectID_mErrorLogbook, this },
		mAEnManager{ mObjectID_mAEnManager }
	{
		showValue("\ncSoftwareManager::cSoftwareManager - Object ID = ", mObjectID);

		create();
		init();
		buildRelations();
		config();
		allocateResources();
	}

	cSoftwareManager::~cSoftwareManager()
	{
		showValue("\ncSoftwareManager::~cSoftwareManager");

		deallocateResources();
		resetRelations();
		deinit();
	}

	void cSoftwareManager::create(void)
	{
		showValue("\ncSoftwareManager::create");
		
		mAEnManager.create();
	}

	void cSoftwareManager::init(void)
	{
		showValue("\ncSoftwareManager::init");
		
		mAEnManager.init();
	}

	void cSoftwareManager::deinit(void)
	{
		showValue("\ncSoftwareManager::deinit");
		
		mAEnManager.deinit();
	}

	void cSoftwareManager::buildRelations(void)
	{
		showValue("\ncSoftwareManager::buildRelations");

		mAEnManager.registerErrorHandler(this);

		mAEnManager.buildRelations();
	}

	void cSoftwareManager::resetRelations(void)
	{
		showValue("\ncSoftwareManager::resetRelations");

		mAEnManager.registerErrorHandler(nullptr);

		mAEnManager.resetRelations();
	}

	void cSoftwareManager::config(void)
	{
		showValue("\ncSoftwareManager::config");
		
		mAEnManager.config(mDataBase.getData());
	}

	void cSoftwareManager::allocateResources(void)
	{
		showValue("\ncSoftwareManager::allocateResources");

		mAEnManager.allocateResources();
	}

	void cSoftwareManager::deallocateResources(void)
	{
		showValue("\ncSoftwareManager::deallocateResources");

		mAEnManager.deallocateResources();
	}

	void cSoftwareManager::start(void)
	{
		showValue("\ncSoftwareManager::start");

		mAEnManager.start();

		executeDiagnostics();
	}

	void cSoftwareManager::taskInit(void)
	{
		showValue("\ncSoftwareManager::taskInit");
	}

	void cSoftwareManager::taskOperational(void)
	{
		showValue("\ncSoftwareManager::taskOperational");
	}

	void cSoftwareManager::executeDiagnostics(void)
	{
		showValue("\ncSoftwareManager::executeDiagnostics");

		Common::Management::tcDiagnostics<mMaxNumberOfAEnDiagnosticEntries> locAEnDiagnostics = mAEnManager.executeDiagnostics();
		uint32_t locIndex = 0;
		
		for (locIndex; locIndex < mMaxNumberOfAEnDiagnosticEntries; ++locIndex)
		{
			mDiagnostics.write(locIndex, locAEnDiagnostics.read(locIndex));
		}

		showValue("\nError logbook SoftwareManager:");
		mErrorLogbook.show();

		showValue("\nSoftware wide diagnostics:");
		mDiagnostics.show();
	}

	void cSoftwareManager::restart(void)
	{
		showValue("\ncSoftwareManager::restart");

		mAEnManager.restart();
	}

	void cSoftwareManager::shutdown(void)
	{
		showValue("\nccSoftwareManager::shutdown");

		mAEnManager.shutdown();
	}

	void cSoftwareManager::handleError(void)
	{
		showValue("\ncSoftwareManager::handleError");
	}

	void cSoftwareManager::recoverError(void)
	{
		showValue("\ncSoftwareManager::recoverError");
	}

	void cSoftwareManager::enterFailSafe(void)
	{
		showValue("\ncSoftwareManager::enterFailSafe");

		mAEnManager.enterFailSafe();
	}

	void cSoftwareManager::notifyError(const uint32_t ObjectID, const Common::Management::ecError Error)
	{
		showValue("\ncSoftwareManager::notifyError");

		Common::Management::cErrorEntry locErrorEntry{ ObjectID, Error };

		mErrorLogbook.addErrorEntry(locErrorEntry);
		mErrorLogbook.show();
	}
}
