#ifndef __cSoftwareManager_HPP__
#define __cSoftwareManager_HPP__

#include "../Common/Management/icbcErrorHandler.hpp"
#include "../Common/Management/tcDiagnostics.hpp"
#include "../Common/Management/cErrorLogbook.hpp"
#include "../AEn/cAEnManager.hpp"

#include "cDataBase.hpp"


namespace Software_Management
{
	class cSoftwareManager : public Common::Management::icbcErrorHandler
	{
		public:
			explicit cSoftwareManager(void);
			virtual ~cSoftwareManager() override;

			void start(void);
			void shutdown(void);

		private:
			void create(void);
			void init(void);
			void deinit(void);
			void buildRelations(void);
			void resetRelations(void);
			void config(void);
			void allocateResources(void);
			void deallocateResources(void);

			void taskInit(void);
			void taskOperational(void);
			void executeDiagnostics(void);

			void restart(void);
			
			void notifyError(const uint32_t ObjectID, const Common::Management::ecError Error) override;
			void handleError(void);
			void recoverError(void);
			void enterFailSafe(void);

		private:
			static const uint32_t mMaxNumberOfAEnDiagnosticEntries = AEn::cAEnManager::getMaxNumberOfDiagnosticEntries();
			static const uint32_t mMaxNumberOfDiagnosticsData = mMaxNumberOfAEnDiagnosticEntries; // + others

			static const uint32_t mObjectID_SystemManager = 1;
			static const uint32_t mObjectID_mDataBase = 2;
			static const uint32_t mObjectID_mDiagnostics = 3;
			static const uint32_t mObjectID_mErrorLogbook = 4;
			static const uint32_t mObjectID_mAEnManager = 5;

			const uint32_t mObjectID;
			cDataBase mDataBase;
			Common::Management::tcDiagnostics<mMaxNumberOfDiagnosticsData> mDiagnostics;
			Common::Management::cErrorLogbook mErrorLogbook;
			AEn::cAEnManager mAEnManager;
	};
}

#endif  // __cSoftwareManager_HPP__
