#include "cSoftwareManager.hpp"

#include "../../Platform/Hardware_Abstraction/imBasicIO.hpp"
using namespace Platform::Hardware_Abstraction;

#include "../Common/Management/icDiagnostics.hpp"
#include "../Common/Management/icErrorLogbook.hpp"
#include "../Common/Management/icAEnManager.hpp"

#include "../Common/Management/cDiagnostics.hpp"
#include "../Common/Management/cErrorLogbook.hpp"
#include "../AEn/cAEnManager.hpp"

namespace Software_Management
{
	cSoftwareManager::cSoftwareManager(void) : 
		mObjectID{ mObjectID_SystemManager },
		mDataBase{ mObjectID_DataBase, this },
		mAEnManagerContainer{ nullptr }
	{
		showValue("\ncSoftwareManager::cSoftwareManager - Object ID = ", mObjectID);

		create();
		init();
		buildRelations();
		config();
		allocateResources();
	}

	cSoftwareManager::~cSoftwareManager()
	{
		showValue("\ncSoftwareManager::~cSoftwareManager");

		deallocateResources();
		resetRelations();
		deinit();
	}

	void cSoftwareManager::create(void)
	{
		showValue("\ncSoftwareManager::create");

		mPtrDiagnostics  = new Common::Management::cDiagnostics{ mObjectID_Diagnostics, this };
		mPtrErrorLogbook = new Common::Management::cErrorLogbook{ mObjectID_ErrorLogbook, this };

		mAEnManagerContainer[mAEnManagerID] = new AEn::cAEnManager{mObjectID_AEnManager};
		// add other AEnManager objects here
		
		// for all AEnManager objects call ...
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->create();
			}
		}
	}

	void cSoftwareManager::init(void)
	{
		showValue("\ncSoftwareManager::init");
		
		//mAEnManagerContainer[mAEnManagerID]->init();

		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->init();
			}
		}
	}

	void cSoftwareManager::deinit(void)
	{
		showValue("\ncSoftwareManager::deinit");
		
		mAEnManagerContainer[mAEnManagerID]->deinit();

		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->deinit();
			}
		}
	}

	void cSoftwareManager::buildRelations(void)
	{
		showValue("\ncSoftwareManager::buildRelations");

		//mAEnManagerContainer[mAEnManagerID]->registerErrorHandler(this);
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->registerErrorHandler(this);
			}
		}

		//mAEnManagerContainer[mAEnManagerID]->buildRelations();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->buildRelations();
			}
		}
	}

	void cSoftwareManager::resetRelations(void)
	{
		showValue("\ncSoftwareManager::resetRelations");

		//mAEnManagerContainer[mAEnManagerID]->registerErrorHandler(nullptr);
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->registerErrorHandler(nullptr);
			}
		}

		//mAEnManagerContainer[mAEnManagerID]->resetRelations();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->resetRelations();
			}
		}
	}

	void cSoftwareManager::config(void)
	{
		showValue("\ncSoftwareManager::config");
		
		//mAEnManagerContainer[mAEnManagerID]->config(mDataBase.getData());
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->config(mDataBase.getData());
			}
		}
	}

	void cSoftwareManager::allocateResources(void)
	{
		showValue("\ncSoftwareManager::allocateResources");

		//mAEnManagerContainer[mAEnManagerID]->allocateResources();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->allocateResources();
			}
		}
	}

	void cSoftwareManager::deallocateResources(void)
	{
		showValue("\ncSoftwareManager::deallocateResources");

		//mAEnManagerContainer[mAEnManagerID]->deallocateResources();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->deallocateResources();
			}
		}
	}

	void cSoftwareManager::start(void)
	{
		showValue("\ncSoftwareManager::start");

		//mAEnManagerContainer[mAEnManagerID]->start();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->start();
			}
		}

		executeDiagnostics();
	}

	void cSoftwareManager::taskInit(void)
	{
		showValue("\ncSoftwareManager::taskInit");
	}

	void cSoftwareManager::taskOperational(void)
	{
		showValue("\ncSoftwareManager::taskOperational");
	}

	void cSoftwareManager::executeDiagnostics(void)
	{
		showValue("\ncSoftwareManager::executeDiagnostics");

		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->executeDiagnostics();
			}
		}

		showValue("\nError logbook SoftwareManager:");
		mPtrErrorLogbook->show();
	}

	void cSoftwareManager::restart(void)
	{
		showValue("\ncSoftwareManager::restart");

		//mAEnManagerContainer[mAEnManagerID]->restart();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->restart();
			}
		}
	}

	void cSoftwareManager::shutdown(void)
	{
		showValue("\nccSoftwareManager::shutdown");

		//mAEnManagerContainer[mAEnManagerID]->shutdown();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->shutdown();
			}
		}
	}

	void cSoftwareManager::handleError(void)
	{
		showValue("\ncSoftwareManager::handleError");
	}

	void cSoftwareManager::recoverError(void)
	{
		showValue("\ncSoftwareManager::recoverError");
	}

	void cSoftwareManager::enterFailSafe(void)
	{
		showValue("\ncSoftwareManager::enterFailSafe");

		//mAEnManagerContainer[mAEnManagerID]->enterFailSafe();
		for (auto&& refElement : mAEnManagerContainer)
		{
			if (refElement != nullptr)
			{
				refElement->enterFailSafe();
			}
		}
	}

	void cSoftwareManager::notifyError(const uint32_t ObjectID, const Common::Management::ecError Error)
	{
		showValue("\ncSoftwareManager::notifyError");

		Common::Management::cErrorEntry locErrorEntry{ ObjectID, Error };

		mPtrErrorLogbook->addErrorEntry(locErrorEntry);
		mPtrErrorLogbook->show();
	}
}
