#ifndef __cSoftwareManager_HPP__
#define __cSoftwareManager_HPP__

#include "../Common/Management/icbcErrorHandler.hpp"
#include "cDataBase.hpp"

#include <array>

namespace Common::Management { class icDiagnostics; }
namespace Common::Management { class icErrorLogbook; }
namespace Common::Management { class icAEnManager; }

namespace Software_Management
{
	class cSoftwareManager : public Common::Management::icbcErrorHandler
	{
		public:
			explicit cSoftwareManager(void);
			~cSoftwareManager() override;

			void start(void);
			void shutdown(void);

		private:
			void create(void);
			void init(void);
			void deinit(void);
			void buildRelations(void);
			void resetRelations(void);
			void config(void);
			void allocateResources(void);
			void deallocateResources(void);

			void taskInit(void);
			void taskOperational(void);
			void executeDiagnostics(void);

			void restart(void);
			
			void notifyError(const uint32_t ObjectID, const Common::Management::ecError Error) override;
			void handleError(void);
			void recoverError(void);
			void enterFailSafe(void);

		private:
			static const uint32_t mObjectID_SystemManager = 1;
			static const uint32_t mObjectID_DataBase = 2;
			static const uint32_t mObjectID_Diagnostics = 3;
			static const uint32_t mObjectID_ErrorLogbook = 4;
			static const uint32_t mObjectID_AEnManager = 5;

			static const uint32_t mMaxNumberOfAEnManager = 10;
			static const uint32_t mAEnManagerID = 0;
			// add other manager IDs here

			const uint32_t mObjectID;
			cDataBase mDataBase;
			Common::Management::icDiagnostics* mPtrDiagnostics;
			Common::Management::icErrorLogbook* mPtrErrorLogbook;
			std::array<Common::Management::icAEnManager*, mMaxNumberOfAEnManager> mAEnManagerContainer;
	};
}

#endif  // __cSoftwareManager_HPP__
